/* schupd.p -- Update the Progress Schema from Roundtable .df files 
This proceedure Updates Progress Schemas.
It also has the capability to run pre-update proceedures to dump any data
that will be deleted from the database due to the schema changes.

The .df files used to update the Schema are complient with Progress 
specifications.  They can be loaded by the Progress Dictionary Process if
necessary.  This is not reccommended as schupdp2.p is both faster and more
disk space efficient than the Progress Dictionary Process.

The update process uses a control file named rtb_idat/schctrl to manage the
update process as a whole.  Each record in the schctrl file specifies the
pre-update, update and post-update requirements of each file.  Status flags
are udpated in each schctrl record as each update process on files are 
completed.  The process can be restarted from the point of failure by
simply re-running the schupd.p.  

This proceedure produces an error file named rtb_ilog/schupd.err.
*/

SESSION:APPL-ALERT-BOXES = TRUE.


/* --- Define New Shared --- */
DEFINE NEW SHARED STREAM Serr.

/* --- Define Locals. --- */
DEFINE STREAM Sin1.
DEFINE STREAM Sout1.
DEFINE STREAM Supdlog.

DEFINE VARIABLE Merror    AS CHARACTER          NO-UNDO.
DEFINE VARIABLE Mfilepos  AS INTEGER            NO-UNDO.
DEFINE VARIABLE Mlastpos  AS INTEGER            NO-UNDO.
DEFINE VARIABLE Mos-error AS INTEGER            NO-UNDO.
DEFINE VARIABLE Mctrlfile AS CHARACTER          NO-UNDO.
DEFINE VARIABLE Mtoken    AS CHARACTER          NO-UNDO. 
DEFINE VARIABLE Mschctrl  AS CHARACTER EXTENT 6 NO-UNDO.
   /* --- Mschctrl[1] = Updatestatus
          Mschctrl[2] = dbase name
          Mschctrl[3] = filename
          Mschctrl[4] = deact-index
          Mschctrl[5] = program code
          Mschctrl[6] = dbdir         --- */

/* --- End of Defines --- */


/* --- Initialize variables. --- */
ASSIGN
   Mctrlfile = "rtb_idat/schctrl".


/* --- Create a log of all actions perfromed during schema update. --- */
OUTPUT TO rtb_ilog/schema.log UNBUFFERED NO-ECHO.

/* --- Create a lock file that the calling process will wait for. --- */
OUTPUT STREAM Sout1 TO rtb_idat/runlock UNBUFFERED NO-ECHO.
OUTPUT STREAM Sout1 CLOSE.

/* --- Remove lock file created by calling process. --- */
IF SEARCH("rtb_idat/strtlock") <> ? THEN 
   OS-DELETE VALUE(SEARCH("rtb_idat/strtlock")).

/* --- Create a log of all errors that occurred during schema update. --- */
OUTPUT STREAM Serr TO rtb_ilog/schupd.err NO-ECHO.

/* --- Create a log of all file actions perfromed during schema update. --- */
OUTPUT STREAM Supdlog TO rtb_ilog/schupd.log UNBUFFERED NO-ECHO.

/* --- Read through the rtb_idat/schctrl file and process tables in that
       file. --- */
ASSIGN
   Mfilepos = 0.

REPEAT:   

   INPUT STREAM Sin1 FROM VALUE(Mctrlfile) NO-ECHO.
   SEEK STREAM Sin1 TO Mfilepos.
   ASSIGN
      Mschctrl = "".
   IMPORT STREAM Sin1 Mschctrl.                    
   ASSIGN
      Mlastpos = Mfilepos
      Mfilepos = SEEK(Sin1).
   INPUT STREAM Sin1 CLOSE.

   IF Mschctrl[1] = "EXPANDED" THEN NEXT.

   IF Mschctrl[1] = "YYY" THEN NEXT. 

   IF Mschctrl[6] = "" THEN
      ASSIGN
         Mschctrl[6] = "rtb_idat/tempdata". 

   IF NOT CONNECTED(Mschctrl[2]) THEN DO:
      PUT STREAM Serr UNFORMATTED
         "Database " Mschctrl[2] " not connected" SKIP.
      IF SEARCH("rtb_idat/runlock") <> ? THEN 
         OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
      RETURN.
   END.  /* if not connected(mschctrl[2]) */

   /* --- Assign DICTDB alias to current database --- */
   IF LDBNAME("DICTDB") <> Mschctrl[2] THEN DO:
      DELETE ALIAS "DICTDB".
      CREATE ALIAS DICTDB FOR DATABASE VALUE(Mschctrl[2]).
   END.  /* if ldbname("dictdb") <> ... */

   OUTPUT STREAM Sout1 TO VALUE(Mctrlfile) NO-ECHO APPEND.

   ASSIGN
      Merror = "".
      
   /* --- Pre-update processing. --- */ 
   IF SUBSTRING(Mschctrl[1],1,1) = "N" THEN DO ON ERROR UNDO, LEAVE:
      ASSIGN
         Merror = "Pre-update Error".
      PUT STREAM Supdlog UNFORMATTED 
         "Pre-Schema Processing for " + Mschctrl[2] + "." + Mschctrl[3] + "..." SKIP.
      FILE-INFO:FILE-NAME = Mschctrl[6].
      IF FILE-INFO:FILE-TYPE = ? THEN
        DO:
          OS-CREATE-DIR VALUE(Mschctrl[6]).
          Mos-error = OS-ERROR.
          IF Mos-error <> 0 THEN
            DO:
              PUT STREAM Serr UNFORMATTED
                "Could not create directory " + Mschctrl[6] + "." SKIP.
              IF SEARCH("rtb_idat/runlock") <> ? THEN 
                OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
              RETURN.
            END.
        END.
      RUN VALUE("rtb_idat/" + Mschctrl[5] + "d.p") (INPUT Mschctrl[6]).
      SEEK STREAM Sout1 TO Mlastpos + 1.
      PUT STREAM Sout1 UNFORMATTED "Y".
      ASSIGN
         Merror = "".
   END.  /* if substring(mschctrl ... */
      
   IF Merror <> "" THEN DO:
      PUT STREAM Serr UNFORMATTED
         "An error occurred during pre-schema update processing for: " +
         Mschctrl[2] + "." + Mschctrl[3] SKIP.
      IF SEARCH("rtb_idat/runlock") <> ? THEN 
         OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
      RETURN.
   END.  /* if merror <> "" */


   /* --- Schema update processing --- */
   IF SUBSTRING(Mschctrl[1],2,1) = "N" THEN DO:
      PUT STREAM Supdlog UNFORMATTED
         "Updating Schema for " + Mschctrl[2] + "." + Mschctrl[3] + "..." SKIP.
      
      RUN rtb_inst/schupd2.p (INPUT  "rtb_idat/" + Mschctrl[5] + ".df",
                              OUTPUT Merror).
      
      IF RETURN-VALUE NE "" AND RETURN-VALUE NE ? THEN
        Merror = RETURN-VALUE.
      IF Merror <> "" THEN DO:
         PUT STREAM Serr UNFORMATTED
             Merror SKIP
             "An error occurred while updating file: " + Mschctrl[2] + "." + Mschctrl[3] SKIP.
         IF SEARCH("rtb_idat/runlock") <> ? THEN 
            OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
         RETURN.
         END.
      SEEK STREAM Sout1 TO Mlastpos + 2.
      PUT STREAM Sout1 UNFORMATTED "Y".
   END. /* if substring(mschctrl[1],2,1) = "n" */

   OUTPUT STREAM Sout1 CLOSE.

END. /* repeat */


/* --- Close log files. --- */
OUTPUT STREAM Serr CLOSE.
OUTPUT STREAM Supdlog CLOSE.
OUTPUT CLOSE.


/* --- Remove lock file so calling process can continue. --- */
IF SEARCH("rtb_idat/runlock") <> ? THEN 
   OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
